local addonName, addon = ...
local E = addon:Eve()

local print = function() end
local INSPECT_TIMEOUT = 1.5

local GuidCache = {
	-- [guid] = { cached datas, timestamp }
}

local CACHE_TIMEOUT = 10 -- seconds to store cached values
local Unit = Unit
local ScanningGUID
local GuidCache = {} -- [guid] = {ilevel, specName, timestamp}
local ActiveGUID
local ActiveUnit = 'mouseover'
hooksecurefunc('NotifyInspect', function(unitID)
	ActiveGUID = nil
	local guid = UnitGUID(unitID)
	if guid and not ScanningGUID then
		ScanningGUID = guid
	end
end)


local function ColorGradient(perc, r1, g1, b1, r2, g2, b2)
	if perc >= 1 then
		local r, g, b = r2, g2, b2 -- select(select('#', ...) - 2, ...)
		return r, g, b
	elseif perc <= 0 then
		local r, g, b = r1, g1, b1
		return r, g, b
	end

	--local num = 2 -- select('#', ...) / 3

	--local segment, relperc = math.modf(perc) --*(num-1))
	--local r1, g1, b1, r2, g2, b2 = select((segment*3)+1, ...)
	

	return r1 + (r2 - r1) * perc, g1 + (g2 - g1) * perc, b1 + (b2 - b1) * perc
end

local function ColorDiff(a, b)
	local diff = a - b
	local perc = diff / 30
	
	local r, g, b
	if perc < 0 then -- higher ilevel than us
		perc = perc * -1
		r, g, b = ColorGradient(perc, 1, 1, 0, 0, 1, 0)
	else
		r, g, b = ColorGradient(perc, 1, 1, 0, 1, 0, 0)
	end
	return r, g, b
end

local ItemLevelPattern1 = ITEM_LEVEL:gsub('%%d', '(%%d+)')
local ItemLevelPattern2 = ITEM_LEVEL_ALT:gsub('([()])', '%%%1'):gsub('%%d', '(%%d+)')

local TwoHanders = {
	-- item types that are two handed, as returned as the 4th result from GetItemInfoInstant
	['INVTYPE_RANGED'] = true,
	['INVTYPE_RANGEDRIGHT'] = true,
	['INVTYPE_2HWEAPON'] = true,
}

local InventorySlots = {}
for i = 1, 17 do
	if i ~= 4 then -- ignore shirt, tabard is 19
		tinsert(InventorySlots, i)
	end
end

local function IsArtifact(itemLink)
	return itemLink:find('|cffe6cc80') -- this is probably a horrible way to find whether it's an artifact
end

local function IsCached(itemLink) -- we can't get the correct level of an artifact until all of its relics have been cached
	local cached = true
	local _, itemID, _, relic1, relic2, relic3 = strsplit(':', itemLink)
	if not GetDetailedItemLevelInfo(itemID) then cached = false end
	if IsArtifact(itemLink) then
		if relic1 and relic1 ~= '' and not GetDetailedItemLevelInfo(relic1) then cached = false end
		if relic2 and relic2 ~= '' and not GetDetailedItemLevelInfo(relic2) then cached = false end
		if relic3 and relic3 ~= '' and not GetDetailedItemLevelInfo(relic3) then cached = false end
	end
	return cached
end

local Sekret = '|Hilvl|h'
local function AddLine(leftText, rightText, r1, g1, b1, r2, g2, b2, dontShow)
	--if GameTooltip:IsVisible() then
		if not r1 then
			r1, g1, b1, r2, g2, b2 = 1, 1, 0, 1, 1, 0
		end
		leftText = Sekret .. leftText
		for i = 2, GameTooltip:NumLines() do
			local leftStr = _G['GameTooltipTextLeft' .. i]
			local text = leftStr and leftStr:GetText()
			if text and text:find(Sekret) then
				-- edit line
				local rightStr = _G['GameTooltipTextRight' .. i]
				leftStr:SetText(leftText)
				rightStr:SetText(rightText)
				if r1 and g1 and b1 then
					leftStr:SetTextColor(r1, g1, b1)
				end
				if r2 and g2 and b2 then
					rightStr:SetTextColor(r2, g2, b2)
				end
				return
			end
		end
		if not dontShow or GameTooltip:IsShown() then
			GameTooltip:AddDoubleLine(leftText, rightText, r1, g1, b1, r2, g2, b2)
			GameTooltip:Show()
		--else
		--	print('tooltip hidden!')
		end
	--end
end

-- OnTooltipSetUnit: NotifyInspect(unit)
-- on INSPECT_READY do
-- for each slot, tooltip:SetInventoryItem(unit, slot)
-- OnTooltipSetItem: if IsCached then update slot item level
-- when all items are accounted for, update tooltip
local SlotCache = {} -- [slot] = itemLevel or false
local ItemCache = {} -- [slot] = itemLink
local TestTips = {}
for i, slot in pairs(InventorySlots) do
	local tip = CreateFrame('GameTooltip', 'AverageItemLevelTooltip' .. slot, nil, 'GameTooltipTemplate')
	tip:SetOwner(WorldFrame, 'ANCHOR_NONE')
	TestTips[slot] = tip
	tip.slot = slot
	tip:SetScript('OnTooltipSetItem', function(self)
		local slot = self.slot
		local _, itemLink = self:GetItem()
		local tipName = self:GetName()
		if itemLink then
			local isCached = IsCached(itemLink)
			if isCached then
				for i = 2, self:NumLines() do
					local str = _G[tipName .. 'TextLeft' .. i]
					local text = str and str:GetText()
					if text then
						local ilevel = text:match(ItemLevelPattern1)
						if not ilevel then
							ilevel = text:match(ItemLevelPattern2)
						end
						if ilevel then
							SlotCache[slot] = ilevel
							ItemCache[slot] = itemLink
						end
					end
				end
			end
		end
		
		local finished = true
		local totalItemLevel = 0
		
		for slot, ilevel in pairs(SlotCache) do
			if not ilevel then
				finished = false
				break
			else
				if slot ~= 16 and slot ~= 17 then
					totalItemLevel = totalItemLevel + ilevel
				end
			end
		end
		
		if finished then
			local weaponLevel = 0
			if SlotCache[16] and SlotCache[17] then -- we have 2 weapons
				if IsArtifact(ItemCache[16]) then -- take the higher of the 2 weapons and double it
					local ilevelMain = SlotCache[16]
					local ilevelOff = SlotCache[17]
					if ilevelMain > ilevelOff then
						totalItemLevel = totalItemLevel + (ilevelMain * 2)
						weaponLevel = ilevelMain
					else
						totalItemLevel = totalItemLevel + (ilevelOff * 2)
						weaponLevel = ilevelOff
					end
				else
					local ilevelMain = SlotCache[16]
					local ilevelOff = SlotCache[17]
					totalItemLevel = totalItemLevel + ilevelMain + ilevelOff
					if ilevelMain > ilevelOff then
						weaponLevel = ilevelMain
					else
						weaponLevel = ilevelOff
					end
				end
			elseif SlotCache[16] then -- main hand only
				local _, _, _, weaponType = GetItemInfoInstant(ItemCache[16])
				local ilevelMain = SlotCache[16]
				weaponLevel = ilevelMain
				if TwoHanders[weaponType] then -- 2 handed, count it twice
					totalItemLevel = totalItemLevel + (ilevelMain * 2)
				else
					totalItemLevel = totalItemLevel + ilevelMain
				end
			elseif SlotCache[17] then -- off hand only?
				local ilevelOff = SlotCache[17]
				totalItemLevel = totalItemLevel + ilevelOff
				weaponLevel = ilevelOff
			end
			
			
			local averageItemLevel = totalItemLevel / 16
			local ourMaxItemLevel, ourEquippedItemLevel = GetAverageItemLevel()
			
			local r1, g1, b1 = ColorDiff(ourEquippedItemLevel, averageItemLevel)
			local ourWeaponMain = GetInventoryItemLink('player', 16)
			local ourWeaponOff = GetInventoryItemLink('player', 17)
			local ourWeaponLevel = 0
			if ourWeaponOff and ourWeaponMain then -- dual wielding
				local mainLevel = GetDetailedItemLevelInfo(ourWeaponMain)
				local offLevel = GetDetailedItemLevelInfo(ourWeaponOff)
				if mainLevel > offLevel then
					ourWeaponLevel = mainLevel
				else
					ourWeaponLevel = offLevel
				end
			elseif ourWeaponMain then
				ourWeaponLevel = GetDetailedItemLevelInfo(ourWeaponMain)
			elseif ourWeaponOff then
				ourWeaponLevel = GetDetailedItemLevelInfo(ourWeaponOff)
			end
			local r2, g2, b2 = ColorDiff(ourWeaponLevel, weaponLevel)
			
			

			local classDisplayName, class = UnitClass(ActiveUnit)
			local colors = class and RAID_CLASS_COLORS[class]
			local specID = GetInspectSpecialization(ActiveUnit)
			local specName -- = GuidCache[guid].specName
			if not specName and specID and specID ~= 0 then
				specID, specName = GetSpecializationInfoByID(specID, UnitSex(ActiveUnit))
				if colors then
					specName = '|c' .. colors.colorStr .. specName .. '|r'
				end
			end
			
			local levelText = format('|cff%2x%2x%2x%.1f|r |cff%2x%2x%2x(%s)|r', r1 * 255, g1 * 255, b1 * 255, averageItemLevel, r2 * 255, g2 * 255, b2 * 255, weaponLevel)
			local guid = UnitGUID(ActiveUnit)
			if guid then
				if not GuidCache[guid] then
					GuidCache[guid] = {}
				end
				GuidCache[guid].specName = specName
				GuidCache[guid].ilevel = averageItemLevel
				GuidCache[guid].weaponLevel = weaponLevel
				GuidCache[guid].levelText = levelText
				GuidCache[guid].timestamp = GetTime()
			end
			
			AddLine(specName or STAT_AVERAGE_ITEM_LEVEL, levelText, 1, 1, 0, r1, g1, b1, true)
		end
	end)
end

local f = CreateFrame('frame', nil, GameTooltip)
local ShouldInspect = false
local LastInspect = 0
f:SetScript('OnUpdate', function(self, elapsed)
	if ShouldInspect and (GetTime() - LastInspect) >= INSPECT_TIMEOUT and CanInspect(ActiveUnit) then
		ShouldInspect = false
		NotifyInspect(ActiveUnit)
	end
end)

hooksecurefunc('NotifyInspect', function()
	LastInspect = GetTime()
end)

local function DoInspect()
	ShouldInspect = true
	AddLine(STAT_AVERAGE_ITEM_LEVEL, 'Loading..', 0.6, 0.6, 0.6, 0.6, 0.6, 0.6)
end

local ActiveGUID
GameTooltip:HookScript('OnTooltipSetUnit', function(self)
	local _, unit = self:GetUnit()
	local guid = unit and UnitGUID(unit)
	if guid then
		local cache = GuidCache[guid]
		--GuidCache[guid].specName = specName
		--GuidCache[guid].ilevel = averageItemLevel
		--GuidCache[guid].weaponLevel = weaponLevel
		if cache and (InCombatLockdown() or (GetTime() - cache.timestamp) <= CACHE_TIMEOUT) then
			--print('cached?')
			AddLine(cache.specName or STAT_AVERAGE_ITEM_LEVEL, cache.levelText, 0.6, 0.6, 0.6, 0.6, 0.6, 0.6)
		elseif not InCombatLockdown() and CanInspect(unit) and not UnitIsDeadOrGhost('player') then
			AddLine(STAT_AVERAGE_ITEM_LEVEL, 'Loading..', 0.6, 0.6, 0.6, 0.6, 0.6, 0.6)
			--GameTooltip:AddDoubleLine(STAT_AVERAGE_ITEM_LEVEL, 'Loading..', 0.6, 0.6, 0.6, 0.6, 0.6, 0.6)
			--GameTooltip:Show()
			if not ActiveGUID then
				--print(GetTime(), unit, (UnitName(unit)), UnitGUID(unit))
				ActiveGUID = UnitGUID(unit)
				ActiveUnit = unit
				--NotifyInspect(unit)
				--ShouldInspect = true
				DoInspect()
				--PrepInspect()
			end
		end
	end
end)

function E:INSPECT_READY(inspectedGUID)
	if not inspectedGUID == ActiveGUID then ActiveGUID = nil end
	if GameTooltip:IsVisible() then
		local _, unit = GameTooltip:GetUnit()
		if unit and UnitIsPlayer(unit) then
			if UnitGUID(unit) == inspectedGUID then
				--local guid = UnitGUID(unit)
				wipe(SlotCache)
				wipe(ItemCache)
				for i, slot in pairs(InventorySlots) do
					if GetInventoryItemTexture(unit, slot) then -- we have an item in this slot
						SlotCache[slot] = false
					end
				end
				for slot in pairs(SlotCache) do
					TestTips[slot]:SetInventoryItem(unit, slot)
				end
			else
				--print(GetTime(), 'reinspecting')
				ClearInspectPlayer()
				--NotifyInspect(unit)
				--ShouldInspect = true
				DoInspect()
			end
		end
	end
	ActiveGUID = nil
end
